#include <stdio.h>
#include <stdlib.h>

#define  BLOCKSIZE  4096


/* --------------------------------------- */
/* --- main function                       */
/* --------------------------------------- */
int main(int argc, char *argv[])
{
  FILE *fp1, *fp2;
  unsigned long length;
  unsigned char  *buffer;
  unsigned long var_long;
  unsigned short var_short;

  if (argc != 3) {
    printf("USAGE: addwavheader inputfile outputfile\n");
    return(-1);
  }

  /* --- open */
  if ((fp1 = fopen(argv[1], "rb")) == NULL) {
    fprintf(stderr, "input file is not found : %s\n", argv[1]);
    return(-1);
  }
  if ((fp2 = fopen(argv[2], "wb")) == NULL) {
    fprintf(stderr, "output file is not open : %s\n", argv[2]);
    fclose(fp1);
    return(-1);
  }

  /* --- data size */
  fseek(fp1, 0, SEEK_END);
  length = ftell(fp1);
  fseek(fp1, 0, SEEK_SET);

  /* --- header */
  buffer = malloc(BLOCKSIZE);
  if (buffer == NULL) {
    printf("memory allocate error.\n");
    goto done;
  }
  buffer[0] = 'R';
  buffer[1] = 'I';
  buffer[2] = 'F';
  buffer[3] = 'F';
  fwrite(buffer, 1, 4, fp2);
  // ファイルサイズ
  var_long = 36 + length;
  fwrite(&var_long, sizeof(long), 1, fp2);

  // WAVE ヘッダ
  buffer[0] = 'W';
  buffer[1] = 'A';
  buffer[2] = 'V';
  buffer[3] = 'E';
  fwrite(buffer, 1, 4, fp2);

  // chunkID (fmt チャンク)
  buffer[0] = 'f';
  buffer[1] = 'm';
  buffer[2] = 't';
  buffer[3] = ' ';
  fwrite(buffer, 1, 4, fp2);
  // chunkSize (fmt チャンクのバイト数 無圧縮 wav は 16)
  var_long = 16;
  fwrite(&var_long, sizeof(long), 1, fp2);
  // wFromatTag (無圧縮 PCM = 1)
  var_short = 0x0001;
  fwrite(&var_short, sizeof(short), 1, fp2);
  // dwChannels (モノラル = 1, ステレオ = 2)
  var_short = 1;
  fwrite(&var_short, 2, 1, fp2);
  // dwSamplesPerSec (サンプリングレート(Hz))
  var_long = 11025;
  fwrite(&var_long, sizeof(long), 1, fp2);
  // wdAvgBytesPerSec (Byte/秒)
  var_long = 11025 * 2;
  fwrite(&var_long, sizeof(long), 1, fp2);
  // wBlockAlign (Byte/サンプル*チャンネル)
  var_short = 4;
  fwrite(&var_short, sizeof(short), 1, fp2);
  // wBitsPerSample (bit/サンプル)
  var_short = 2 * 8;
  fwrite(&var_short, sizeof(short), 1, fp2);

  // chunkID (data チャンク)
  buffer[0] = 'd';
  buffer[1] = 'a';
  buffer[2] = 't';
  buffer[3] = 'a';
  fwrite(buffer, 1, 4, fp2);
  // chunkSize (データ長 Byte)
  fwrite(&length, 4, 1, fp2);

  // ヘッダ (44 Byte) 書き込みここまで
  if (ftell(fp2) != 44) {
    fprintf(stderr, "%s : wav header write error\n", argv[2]);
    exit(1);
  }

  /* --- data body */
  while((length = fread(buffer, 1, BLOCKSIZE, fp1)) > 0) {
    fwrite(buffer, 1, length, fp2);
  }

  /* --- close :*/
 done:
  fclose(fp1);
  fclose(fp2);
  return(0);
}
